/*
 * Copyright 2004 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Header:$
 */

package org.apache.beehive.netui.tools.testrecorder.shared;

import java.util.List;
import java.util.ArrayList;
import java.util.Collections;

/**
 * User: ozzy
 * Date: Jun 15, 2004
 * Time: 9:19:12 AM
 */
public class PlaybackSessionBean extends RecordSessionBean {

    private static final Logger log = Logger.getInstance( PlaybackSessionBean.class );

    private int recordedTestCount = -1;
    private int passedCount = 0;
    private int failedCount = 0;
    private List testResults;

    public PlaybackSessionBean( String sessionName ) {
        this( sessionName, false );
    }

    public PlaybackSessionBean( String sessionName, boolean error ) {
        super( sessionName );
        setError( error );
        testResults = new ArrayList();
    }

    public int getRecordedTestCount() {
        return recordedTestCount;
    }

    public void setRecordedTestCount( int recordedTestCount ) {
        this.recordedTestCount = recordedTestCount;
    }

    public int getTestCount() {
        return testResults.size();
    }

    public TestResults getTestResults( int index ) {
        return (TestResults) testResults.get( index );
    }

    public List getTestResults() {
        return Collections.unmodifiableList( testResults );
    }

    public void addTestResults( TestResults results ) {
        if ( results.isTestPassed() ) {
            incrementPassedCount();
        }
        else {
            incrementFailedCount();
        }
        testResults.add( results );
    }

    public boolean isError() {
        if ( error ) {
            return error;
        }
        return isErrorInternal();
    }

    private boolean isErrorInternal() {
        boolean rtnVal = false;
        if ( getRecordedTestCount() < 1 ) {
            if ( log.isDebugEnabled() ) {
                log.debug( "Invalid recorded test count(" + getRecordedTestCount() + ")" );
            }
            rtnVal = true;
        }
        else if ( ( getTestCount() ) != getRecordedTestCount() ) {
            if ( log.isDebugEnabled() ) {
                log.debug( "executed count(" + getTestCount() + ") does not equal recorded test count(" +
                        getRecordedTestCount() + ")" );
            }
            rtnVal = true;
        }
        return rtnVal;
    }

    public String getStatus() {
        if ( isError() ) {
            return Constants.ERROR;
        }
        if ( isSessionPassed() ) {
            return Constants.PASS;
        }
        return Constants.FAIL;
    }

    public boolean isSessionPassed() {
        if ( isError() ) {
            return false;
        }
        if ( passedCount != getRecordedTestCount() ) {
            return false;
        }
        return true;
    }

    public int incrementPassedCount() {
        return ++passedCount;
    }

    public int getPassedCount() {
        return passedCount;
    }

    public int incrementFailedCount() {
        return ++failedCount;
    }

    public int getFailedCount() {
        return failedCount;
    }

}
